package pi7;

import java.util.ArrayList;
import java.util.List;
import org.antlr.v4.runtime.*;
import org.antlr.v4.runtime.tree.ParseTree;


public class GerenciadorProgramas {
    private List<LinhaPrograma> programa = new ArrayList<LinhaPrograma>();
    private List<String> errorList = null;
    
    public void loadProgram(String filename) throws Exception {
        ANTLRFileStream f = new ANTLRFileStream(filename);
        GcodeLexer lex = new GcodeLexer(f);
        CommonTokenStream tokens = new CommonTokenStream(lex);
        GcodeParser parser = new GcodeParser(tokens);
        ErrorHandler errorHandler = new ErrorHandler();
        parser.addErrorListener(errorHandler);
        errorList = new ArrayList<String>();
        ParseTree tree = parser.def();
        if ((tree != null)&&(errorList.isEmpty())) {
            GcodeProcessor p = new GcodeProcessor(programa);
            p.visit(tree);
        } else {
            throw new Exception(errorList.get(0));
        }
    } // loadProgram

    public List<String> obterListagem() {
        List<String> listagem = new ArrayList<String>();
        String linha;
        for(LinhaPrograma p : programa) {
           if (p.codigoG != null) {
                linha = "N" + p.numeroLinha + " " + p.codigoG + " X" + p.x + " Y" + p.y;
           } else {
                linha = "N" + p.numeroLinha + " " + p.codigoM;               
           }
           listagem.add(linha); 
        } //for
        return listagem;
    } //obterListagem
    
    class ErrorHandler extends BaseErrorListener {

        @Override
        public void syntaxError(Recognizer<?, ?> recognizer, Object offendingSymbol, int line, int charPositionInLine, String msg, RecognitionException e) {
          errorList.add("Erro na linha " + line + " posicao " + charPositionInLine + ":" + msg);
        } //syntaxError
    
    } // ErroHandler
} //GerenciadorProgramas
